package premium_store.util.error;

import org.springframework.context.support.DefaultMessageSourceResolvable;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestController;
import premium_store.service.exception.ClientNotFoundException;
import premium_store.service.exception.LevelNotFoundException;
import premium_store.service.exception.NationNotFoundException;
import premium_store.service.exception.TankNotFoundException;
import premium_store.util.validation.ValidationException;

import java.util.stream.Collectors;

//контроллер для обработки разнообразных ошибок при работе с запросами к БД
@ControllerAdvice(annotations = RestController.class)
public class AdviceController {
    //метод handleException будет вызываться при возникновении исключений типа LevelNotFoundException и т. д.
    @ExceptionHandler({
            LevelNotFoundException.class,
            NationNotFoundException.class,
            ClientNotFoundException.class,
            TankNotFoundException.class,
            ValidationException.class
    })
    public ResponseEntity<Object> handleException(Throwable e) {
        return new ResponseEntity<>(e.getMessage(), HttpStatus.BAD_REQUEST);
    }

    //при исключении автоматической валидации данных
    @ExceptionHandler(MethodArgumentNotValidException.class)
    public ResponseEntity<Object> handleBindException(MethodArgumentNotValidException e) {
        final ValidationException validationException = new ValidationException(
                e.getBindingResult().getAllErrors().stream()
                        .map(DefaultMessageSourceResolvable::getDefaultMessage)
                        .collect(Collectors.toSet()));

        return handleException(validationException);
    }

    //обработка неизвестных ошибок
    @ExceptionHandler(Exception.class)
    public ResponseEntity<Object> handleUnknownException(Throwable e) {
        e.printStackTrace();

        return new ResponseEntity<>(e.getMessage(), HttpStatus.INTERNAL_SERVER_ERROR);
    }
}